#include <math.h>
#include "Line.h"

Line::Line() : Shape() {
  // Set the default line coordinates and style.
  x1 = x2 = y1 = y2 = 0;
  handle_status = 0;
}

Line::Line(double r, double g, double b, double thickness, double x1,
	   double y1, double x2, double y2) : Shape(r,g,b,thickness) {
  // Set the line style and its coordinates according to the parameters.
  setcoords(x1,y1,x2,y2);
  handle_status = 0;
}

Line::Line(Line *line) {
  setcoords(line->get_x1(),line->get_y1(),line->get_x2(),line->get_y2());
  setstyle(line->get_r(),line->get_g(),line->get_b(),line->get_thickness());
  handle_status = 0;
}

void Line::setcoords(double x1, double y1, double x2, double y2) {
  // Set the line coordinates according to the parameters.
  this->x1 = x1;
  this->y1 = y1;
  this->x2 = x2;
  this->y2 = y2;
}

int Line::mouse_on(double mousex, double mousey ,int handlesize){
  // Determine the normal line equation from (mousex,mousey) to the line.
  double m = ((double)y2 - y1) / (x2 - x1);
  double b1 = (double)y1 - m * x1;
  double b2 = mousey + (1/m) * mousex;
  double x = (b2 - b1) / (m + 1/m);
  double y = (b1 - b2) / (m*m + 1) + b2;
  double middlex = (x1+x2) / 2;
  double middley = (y1+y2) / 2;
  double in_boundary = sqrt(pow(x1-middlex,2) + pow(y1-middley,2)) - handlesize;
  // Check for boundary cases (line is horizontal or vertical)
  if (x1 == x2) {
    x = x1;
    y = mousey;
  } else if (y1 == y2) {
    x = mousex;
    y = y1;
  }
  
  // Check if the mouse is near the line.
  if (pow(x-mousex,2) + pow(y-mousey,2) < pow(handlesize,2)) {
    // Mouse is near the inside of the line.
    if (pow(x-middlex,2) + pow(y-middley,2) <= pow(in_boundary,2)) {
      return 3;
      // Mouse is near first endpoint
    } else if (pow(x-x1,2) + pow(y-y1,2) < pow(handlesize,2)) {
      return 1;
      // Mouse is near second endpoint
    } else if (pow(x-x2,2) + pow(y-y2,2) < pow(handlesize,2)) {
      return 2;
      // Mouse is not near the line
    } else {
      return 0;
    }
  } else {
    // Mouse is not near the line
    return 0;
  }
}

void Line::draw_handle(int x, int y, int handlesize) {
  int d = handlesize / 2;
  glColor3f(get_r(),get_g(),get_b());
  glBegin(GL_POLYGON);
  glVertex2i(x - d, y - d);
  glVertex2i(x + d, y - d);
  glVertex2i(x + d, y + d);
  glVertex2i(x - d, y + d);
  glEnd();
}

void Line::draw_arrow(double mag) {
  glPointSize(get_thickness()*4);
  glBegin(GL_POINTS);
  glVertex2f(x2*mag,y2*mag);
  glEnd();
}

void Line::draw(double mag) {
  // Activate the current line style and draw the line.
  activatestyle();
  glBegin(GL_LINES);
  glVertex2f(x1*mag,y1*mag);
  glVertex2f(x2*mag,y2*mag);
  glEnd();
  draw_arrow(mag);
  if (handles_on()) {
    draw_handle(int(x1*mag),int(y1*mag),int(10*mag));
    draw_handle(int(x2*mag),int(y2*mag),int(10*mag));
  }
}

int Line::get_type() {
  return LINE;
}

void Line::read(fstream& f) {
  // First read the line style and then its coordinates from a file.
  readstyle(f);
  f.read((char *)&x1,sizeof(double));
  f.read((char *)&x2,sizeof(double));
  f.read((char *)&y1,sizeof(double));
  f.read((char *)&y2,sizeof(double));
}

void Line::write(fstream& f) {
  // First write the line style and then its coordinates to a file.
  writestyle(f);
  f.write((char *)&x1,sizeof(double));
  f.write((char *)&x2,sizeof(double));
  f.write((char *)&y1,sizeof(double));
  f.write((char *)&y2,sizeof(double));
}
  
