#ifndef SHAPE_H
#define SHAPE_H

#include <GL/gl.h>
#include <FL/gl.h>
#include <fstream.h>

// Shape type definitions
#define LINE 0
#define CIRCLE 1

// Base class for drawing shapes.
class Shape {
  private:
    // The color and line thickness of the shape.
    double r_color, g_color, b_color;
    double thickness;
  
  public:

    // Shape default constructor.
    // Sets the color to white and line width to 1.
    Shape() {
      setstyle(1, 1, 1, 1);
    }

    // Shape constructor.
    // Sets the color and line thickness to the specified values.
    Shape(double r, double g, double b, double thickness) {
      setstyle(r,g,b,thickness);
    }

    // Sets the shape style according to the parameter values.
    void setstyle(double r, double g, double b, double thickness) {
      r_color = r;
      g_color = g;
      b_color = b;
      this->thickness = thickness;
    }

    // Reads a shape style from a stream f.
    void readstyle(fstream& f) {
      f.read((char *)&r_color,sizeof(double));
      f.read((char *)&g_color,sizeof(double));
      f.read((char *)&b_color,sizeof(double));
      f.read((char *)&thickness,sizeof(double));
    }

    // Writes the shape style to a stream f.
    void writestyle(fstream& f) {
      f.write((char *)&r_color,sizeof(double));
      f.write((char *)&g_color,sizeof(double));
      f.write((char *)&b_color,sizeof(double));
      f.write((char *)&thickness,sizeof(double));
    }

    // Activates the current shape style.
    // Sets the specified color and line width.
    void activatestyle() {
      glColor3f(r_color,g_color,b_color);
      glLineWidth(thickness);
    }

    // The following functions return each of the current shape style
    // components.
    double get_r() { return r_color; }
    double get_g() { return g_color; }
    double get_b() { return b_color; }
    double get_thickness() { return thickness; }

    // Draws a red cross of specified size in the coordinates (x,y).
    void draw_cross(int x, int y, int size) {
      glColor3f(1,0,0);
      glLineWidth(1);
      glBegin(GL_LINES);
        glVertex2i(x,y+size);
        glVertex2i(x,y-size);
        glVertex2i(x-size,y);
        glVertex2i(x+size,y);
      glEnd();
    }

    // Draws the current shape.
    virtual void draw(double mag) = 0;
    // Returns the current shape type.
    virtual int get_type() = 0;
    // Read the current shape from a stream f.
    virtual void read(fstream& f) = 0;
    // Write the current shape to a stream f.
    virtual void write(fstream& f) = 0;
};
  
#endif
