#ifndef IMDRAW_H
#define IMDRAW_H

#include <FL/Fl.h>
#include <FL/Fl_Window.h>

#include <FL/gl.h>
#include <FL/Fl_Gl_Window.h>
#include <list>
#include "Shape.h"
#include "Texture.h"

using namespace std;

class ImDraw : public Fl_Gl_Window {  
 public:
  /* The class constructor */
    ImDraw(int x,int y,int w,int h,const char *l=0);
    // Sets the partner window for the current window
    void mypartner(ImDraw *partner);
    /* The widget class draw() override.
     * The draw() function initialize Gl for another round of
     * drawing then calls specialized functions for drawing each
     * of the entities displayed in the cube view.     */    
    void draw();
    // Set the background image for the window
    void set(VImage im);
    // Handle input events
    int handle(int event);
    // Handle resizing the window
    void resize(int x, int y, int w, int h);

    /* Set the current state to drawing lines. */
    void set_line();
    /* Set the current state to selecting shapes. */
    void set_select();
    /* Load an image from a file.
     * Returns the chosen filename. */
    char* Load();
    /* Save an image to a file.
     * Returns the chosen filename. */
    char* Save();
    /* Delete the current list of shapes. */
    void delete_shapes();
    /* Clear the canvas and prepare for a new drawing. */
    void New();
    /* Adds a shape to this window. */
    void addshape(Shape* shape);
    /* Deletes a shape from this window. */
    void deleteshape(int index);

    VImage* getimage();
    list<Shape *> *getshapes();

  private:
    /* (x1,y1) - the coordinates of the mouse at first button push.
     * (x2,y2) - the coordinates of the mouse where it is currently draged. */
    double x1,y1,x2,y2;

    /* State of the left mouse button. 1 iff it is pressed. */
    int mousedown;

    /* The current shape type. LINE or CIRCLE. */
    int shapetype;

    /* The current shape (the shape in process of being drawn). */
    Shape* curshape;

    /* A list of completed shapes that were drawn before. */
    list<Shape *> shapes;

    // A flag that indicates if there is a line selected
    int is_selected;
    // A reference to the selected line
    list<Shape *>::iterator selected;
    // A type of selection: Either one of the line edges or the inside.
    int what_selected;
    // The relative position of the mouse to the line points when it was
    // selected.
    double delta_x1, delta_y1, delta_x2, delta_y2;

    // Width and Height of the window.
    int width, height;
    // Original width and height of the image.
    int im_height, im_width;
    // The aspect ratio of the image
    double aspect_ratio;
    // The texture object for the window.
    Texture *imTexture;
    // The image object for the window.
    VImage *im;
    // The image attributes.
    VAttrList attrs;
    
    ImDraw* partner;
};

#endif
